// -*- C++ -*-

#ifndef _MMAPWRAPPER_H_
#define _MMAPWRAPPER_H_

#if defined(_WIN32)
#include <windows.h>
#else
// UNIX
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <map>
#endif

#include <stdio.h> // FIX ME

#if HL_EXECUTABLE_HEAP
#define HL_MMAP_PROTECTION_MASK (PROT_READ | PROT_WRITE | PROT_EXEC)
#else
#define HL_MMAP_PROTECTION_MASK (PROT_READ | PROT_WRITE)
#endif

#if !defined(MAP_ANONYMOUS) && defined(MAP_ANON)
#define MAP_ANONYMOUS MAP_ANON
#endif

class MmapWrapper {
 public:

#if defined(_WIN32) 
  
  // Microsoft Windows aligns all memory to a 64K boundary.
  enum { Alignment = 64 * 1024 };

  static void * map (size_t sz) {
    void * ptr;
#if HL_EXECUTABLE_HEAP
    const int permflags = PAGE_EXECUTE_READWRITE;
#else
    const int permflags = PAGE_READWRITE;
#endif
    ptr = VirtualAlloc (NULL, sz, MEM_RESERVE | MEM_COMMIT | MEM_TOP_DOWN, permflags);
    return  ptr;
  }
  
  static void unmap (void * ptr, size_t sz) {
    VirtualFree (ptr, sz, MEM_RELEASE);
  }

#else

#if defined(__SVR4)
  // Solaris aligns all memory to a 64K boundary.
  enum { Alignment = 64 * 1024 };
#else
  // Linux and most other operating systems align memory to a 4K boundary.
  enum { Alignment = 4 * 1024 };
#endif

  static void * map (size_t sz) {

    if (sz == 0) {
      abort();
    }

    void * ptr;
#if defined(MAP_ALIGN) && defined(MAP_ANON)
    // Request memory aligned to the Alignment value above.
    ptr = mmap ((char *) Alignment, sz, HL_MMAP_PROTECTION_MASK, MAP_PRIVATE | MAP_ALIGN | MAP_ANON, -1, 0);
#elif !defined(MAP_ANONYMOUS)
    static int fd = ::open ("/dev/zero", O_RDWR);
    ptr = mmap (NULL, sz, HL_MMAP_PROTECTION_MASK, MAP_PRIVATE, fd, 0);
#else
    ptr = mmap (NULL, sz, HL_MMAP_PROTECTION_MASK, MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
#endif
    
    return ptr;
  }

  static void unmap (void * ptr, size_t sz) {
    munmap (reinterpret_cast<char *>(ptr), sz);
  }
   
#endif

};


#endif
